#! /bin/bash

username=$USER
dateFormat='+%A %T %d %B %Y'
notifExpireTime=10000
sleepTime=2
searchPts=false

print_help() {
cat <<EOF
Usage: $(basename $0) [KEY]...
Shows notification with info about previous login.
It uses 'last' util to get info.

  -d, --dateformat <format>  date format using in login message
                             use syntax for 'date' util
  -e, --expire-time <number> duration, in milliseconds, for the notification to appear on screen
                             default $notifExpireTime
  -s, --sleep <number>       duration, in seconds, for showing delay
                             default $sleepTime
  -t, --terminal             consider terminal logins
                             default $searchPts
  -u, --user <user>          show notification for <user>
                             default user is \$USER
  -h, --help                 show this help

EOF
}

print_help_ru() {
cat <<EOF
Использование: $(basename $0) [КЛЮЧ]...
Отображает уведомление с информацией о предыдущем входе.
Для получения информации используется утилита 'last'.

  -d, --dateformat <format>  формат даты для сообщения о входе
                             используйте синтаксис утилиты 'date'
  -e, --expire-time <number> длительность отображения уведомления на экране, в миллисекундах
                             по умолчанию $notifExpireTime
  -s, --sleep <number>       длительность задержки перед отображением, в секундах
                             по умолчанию $sleepTime
  -t, --terminal             учитывать терминальные входы
                             по умолчанию $searchPts
  -u, --user <user>          отобразить уведомление для <user>
                             по умолчанию используется \$USER
  -h, --help                 показать эту справку и выйти

EOF
}

check_second_arg() {
if [ ! -n "$2" ]; then
    if [ "$LANG" = "ru_RU.UTF-8" ]; then
        echo "$1: пропущен операнд"
        echo "Смотрите $(basename $0) --help"
    else
        echo "$1: missing argument"
        echo "See $(basename $0) --help"
    fi
    
    exit 1
fi
}

while [ -n "$1" ]; do
    case "$1" in
        --dateformat|-d)
            check_second_arg dateformat $2
            dateFormat="$2"
            shift; shift
            ;;
        --expire-time|-e)
            check_second_arg expire-time $2
            notifExpireTime="$2"
            shift; shift
            ;;
        --sleep|-s)
            check_second_arg sleep $2
            sleepTime="$2"
            shift; shift
            ;;
        --terminal|-t)
            searchPts=true
            shift
            ;;
        --user|-u)
            check_second_arg user $2
            username="$2"
            shift; shift
            ;;
        --help|-h|*)
            [ "$LANG" = "ru_RU.UTF-8" ] && print_help_ru || print_help
            exit 1
            ;;
    esac
done

# without sleep sometimes it cannot be shown
sleep $sleepTime

notifyCmd="/usr/bin/notify-send --expire-time=$notifExpireTime --icon=preferences-system-login --urgency=low"
wtmpOld='/var/log/wtmp.1'
lastGrepFilter='(still running)|(gone - no)|(still logged)|(pts/)'
if [ "$searchPts" = true ]; then
    lastGrepFilter='(still running)|(gone - no)|(still logged)'
fi

authlog="/var/log/auth.log"
authlog1="/var/log/auth.log.1"

stringTitle=" "
stringSubTitle="Previous user login <b>$username</b>:"
stringNotFound="Your last login record was not found!"
stringLastLogin="Date and time of last login:"
stringSessionTime="Session lasted"
stringLessThanMin="less than a minute"
stringDay="d"
stringHour="h"
stringMin="m"
stringUnknownTime="&#60;unknown&#62;"
stringLoginsCount="Number of logins (since"
stringSucceeded="succeeded"
stringFailed="failed"

# translate strings
if [ "$LANG" = "ru_RU.UTF-8" ]; then
    stringTitle=" "
    stringSubTitle="Предыдущий вход пользователя <b>$username</b>:"
    stringNotFound="Запись вашего последнего входа не найдена!"
    stringLastLogin="Дата и время последнего входа в систему:"
    stringSessionTime="Сессия продлилась"
    stringLessThanMin="меньше минуты"
    stringDay="д"
    stringHour="ч"
    stringMin="м"
    stringUnknownTime="&#60;неизвестно&#62;"
    stringLoginsCount="Количество входов (с"
    stringSucceeded="успешно"
    stringFailed="неуспешно"
fi

# get prev login data
lastData=`/usr/bin/last --nohostname --time-format iso $username \
    | grep ' - ' \
    | grep --invert-match -E "$lastGrepFilter" \
    | head -n 1`

# debug
#lastData="q        pts/0        2019-07-24T15:04:17+0300 - 2019-07-25T15:06:32+0300 (1+00:02)"

# look for wtmp.1 file if data is empty
if [ -z "$lastData" ] && [ -f "$wtmpOld" ]; then
    lastData=`/usr/bin/last --file $wtmpOld --nohostname --time-format iso $username \
        | grep ' - ' | grep --invert-match -E "$lastGrepFilter" | head -n 1`
fi

# exit if data not found
if [ -z "$lastData" ]; then
    $notifyCmd "$stringTitle" "$stringSubTitle\n$stringNotFound"
    exit 0
fi

# get login date
lastLoginStart=`echo $lastData | awk '{ print $3 }'`
lastLoginStart=`date --date=$lastLoginStart "$dateFormat"`

# uncomment this if you need logout date with more readable session time (with seconds)
# date -d '$date' +%s; date -d '@secs'
# lastLoginEnd can be: still running, still logged in, down, crash, gone - no logout
#lastLoginEnd=`echo $lastData | awk '{ print $5 }'`
#lastLoginEnd=`date --date=lastLoginEnd '+%T %A %d %B %Y' 2>/dev/null`
#if [ $? -ne 0 ]; then lastLoginEnd=''; fi

# get session time
lastLoginTime=`echo $lastData | awk '{ print $6 }'`
lastLoginTime=${lastLoginTime#"("}
lastLoginTime=${lastLoginTime%")"}
if [ -z "$lastLoginTime" ]; then
    lastLoginTime=$stringUnknownTime
elif [ "$lastLoginTime" = "00:00" ]; then
    lastLoginTime=$stringLessThanMin
else
    lastLoginTime=`echo $lastLoginTime | sed -r "s/\+/$stringDay /g"`
    lastLoginTime=`echo $lastLoginTime | sed -r "s/:/$stringHour /g"`
    lastLoginTime="$lastLoginTime$stringMin"
fi

# count good/bad logins auth.log
#grep -E "(login[[0-9]+]:|fly-dm:).*session opened for user $username" "$authlog"
if [ -f "$authlog" ]; then
    goodLoginsCount=$(grep -cE "(login[[0-9]+]:|fly-dm: ).*session opened for user $username" "$authlog")
    badLoginsCount=$(grep -cE "FAILED LOGIN.*FOR '$username'" "$authlog")
    if [ "$badLoginsCount" = "0" ]; then
	    badLoginsCount=$(grep -cE "authentication failure.* user=$username" "$authlog")
        OLD_IFS=${IFS}
        IFS=$'\n'
        badLoginsList=$(grep -E "authentication failure.* user=$username" "$authlog" | awk '{print $1,$2,$3}')
        goodLoginsList=$(grep -E "authentication success.* user=$username" "$authlog" | awk '{print $1,$2,$3}')
        for badLine in ${badLoginsList}; do
            badDate=$(date -d "${badLine}" +%s)
            for goodLine in ${goodLoginsList}; do
                goodDate=$(date -d "${goodLine}" +%s)
                if [ $goodDate -ge $badDate -a $goodDate -le $(( $badDate + 2 )) ]; then
                    (( badLoginsCount-- ))
                fi
            done
        done
        IFS=${OLD_IFS}
    fi

    countLoginsFromDate=$(head -n 1 $authlog | awk '{print $1 " " $2 " " $3; exit}')
fi

# count good/bad logins auth.log.1
if [ -f "$authlog1" ]; then
    goodLoginsCount1=$(grep -cE "(login[[0-9]+]:|fly-dm: ).*session opened for user $username" "$authlog1")
    badLoginsCount1=$(grep -cE "FAILED LOGIN.*FOR '$username'" "$authlog1")
    if [ "$badLoginsCount1" = "0" ]; then
        badLoginsCount1=$(grep -cE "authentication failure.* user=$username" "$authlog1")
        OLD_IFS=${IFS}
        IFS=$'\n'
        badLoginsList1=$(grep -E "authentication failure.* user=$username" "$authlog1" | awk '{print $1,$2,$3}')
        goodLoginsList1=$(grep -E "authentication success.* user=$username" "$authlog1" | awk '{print $1,$2,$3}')
        for badLine in ${badLoginsList1}; do
            badDate=$(date -d "${badLine}" +%s)
            for goodLine in ${goodLoginsList1}; do
                goodDate=$(date -d "${goodLine}" +%s)
                if [ $goodDate -ge $badDate -a $goodDate -le $(( $badDate + 2 )) ]; then
                    (( badLoginsCount1-- ))
                fi
            done
        done
        IFS=${OLD_IFS}
    fi
    goodLoginsCount=$(( $goodLoginsCount + $goodLoginsCount1 ))
    badLoginsCount=$(( $badLoginsCount + $badLoginsCount1 ))
    countLoginsFromDate=$(head -n 1 $authlog1 | awk '{print $1 " " $2 " " $3; exit}')
fi

if [ -z "$countLoginsFromDate" ]; then
    goodLoginsCount=0
    badLoginsCount=0
    countLoginsFromDate=$(date '+%d %B %T')
else
    countLoginsFromDate=$(date --date="$countLoginsFromDate" '+%d %B %T')
fi

# show message
$notifyCmd "$stringTitle" \
    "$stringSubTitle
    $stringLastLogin <b>$lastLoginStart</b>
    $stringSessionTime <b>$lastLoginTime</b>
    $stringLoginsCount $countLoginsFromDate): $stringSucceeded: <b>$goodLoginsCount</b>, $stringFailed: <b>$badLoginsCount</b>"
